using NUnit.Framework;
using Rhino.Mocks;
using Rhino.Mocks.Constraints;

namespace RhinoMocks35Beta
{
    [TestFixture]
    public class OldStyleTests
    {
        [Test]
        public void TakePaymentViaPaymentProcessorUsingMockService()
        {
            MockRepository mocks = new MockRepository();
            IPaymentProcessing mockProxy = mocks.CreateMock<IPaymentProcessing>();

            using (mocks.Record())
            {
                Expect.Call(mockProxy.TakePayment(1, 1, 10.0)).IgnoreArguments()
                                     .Return(true);
            }

            using (mocks.Playback())
            {
                PaymentProcessor pp = new PaymentProcessor(mockProxy);
                bool result = pp.TakePayment(1, 1, 10.0);
                Assert.IsTrue(result);
            }
        }
    }

    [TestFixture]
    public class StrictSyntax
    {
        [Test]
        public void TakePaymentViaPaymentProcessorUsingMockService()
        {
            MockRepository mocks = new MockRepository();

            IPaymentProcessing mockProxy = mocks.StrictMock<IPaymentProcessing>();

            using (mocks.Record())
            {
                Expect.Call(mockProxy.TakePayment(1, 1, 10.0))
                                     .IgnoreArguments()
                                     .Return(true);
            }

            using (mocks.Playback())
            {
                PaymentProcessor pp = new PaymentProcessor(mockProxy);
                bool result = pp.TakePayment(1, 1, 10.0);
                Assert.IsTrue(result);
            }
        }
    }
    
    [TestFixture]
    public class AAASyntax
    {
        [Test]
        public void GenerateMock_TakePaymentViaPaymentProcessorUsingMockService()
        {
            IPaymentProcessing mockProxy = MockRepository.GenerateMock<IPaymentProcessing>();

            //mockProxy.Expect(x => x.TakePayment(1, 1, 10.0)).Constraints(Is.Equal(true));  = failed: System.InvalidOperationException : The number of constraints is not the same as the number of the method's parameters!
            mockProxy.Expect(x => x.TakePayment(1, 1, 10.0))
                     .Constraints(Is.Equal(1), Is.Equal(1), Is.Equal(10.0))  //Constraints on the arguments which must be passed into the method
                     .Return(true); //What the method will return when called

            PaymentProcessor pp = new PaymentProcessor(mockProxy);
            bool result = pp.TakePayment(1, 1, 10.0);
            Assert.IsTrue(result);

            mockProxy.VerifyAllExpectations();
        }

        [Test]
        public void GenerateStub_TakePaymentViaPaymentProcessorUsingMockService()
        {
            IPaymentProcessing stubProxy = MockRepository.GenerateStub<IPaymentProcessing>();
            stubProxy.Stub(action => action.TakePayment(1, 1, 10.0)).Return(true);

            PaymentProcessor pp = new PaymentProcessor(stubProxy);
            bool result = pp.TakePayment(1, 1, 10.0);
            Assert.IsTrue(result);
        }

        [Test]
        public void GenerateStub_AssertWasCalled_TakePaymentViaPaymentProcessorUsingMockService()
        {
            IPaymentProcessing stubProxy = MockRepository.GenerateStub<IPaymentProcessing>();
            stubProxy.Stub(action => action.TakePayment(1, 1, 10.0)).Return(true);

            PaymentProcessor pp = new PaymentProcessor(stubProxy);
            bool result = pp.TakePayment(1, 1, 10.0);
            Assert.IsTrue(result);

            //stubProxy.AssertWasCalled(x => x.TakePayment(1, 1, 10.00)); //Expectd that IPaymentProcessing.TakePayment(1, 1, 10); would be called, but is was it was not found on the actual calls made on the mocked object            //stubProxy.AssertWasCalled(x => x.TakePayment(1, 1, 10.00)); //Expectd that IPaymentProcessing.TakePayment(1, 1, 10); would be called, but is was it was not found on the actual calls made on the mocked object
            //stubProxy.AssertWasCalled(x => x.TakePayment(Arg<int>.Matches(p => p == 1 || p == 2), 1, 10.00));
            stubProxy.AssertWasCalled(x => x.TakePayment(1, 1, 10.00));
        }
    }
}